@@ -25,8 +25,8 @@ use std::sync::mpsc::Sender as ThreadOut;
 #[cfg(feature = "std")]
 use std::sync::mpsc::{channel, Receiver};
 
-#[cfg(feature = "std")]
-use std::convert::TryFrom;
+use core::convert::TryFrom;
+use core::ops::Deref;
 
 use codec::{Decode, Encode};
 
@@ -110,23 +110,23 @@ pub type Balance = u128;
 #[derive(Clone, Eq, PartialEq, Default, Debug, Encode, Decode)]
 pub struct AccountDataGen<Balance> {
     /// Non-reserved part of the balance. There may still be restrictions on this, but it is the
-	/// total pool what may in principle be transferred, reserved and used for tipping.
-	///
-	/// This is the only balance that matters in terms of most operations on tokens. It
-	/// alone is used to determine the balance when in the contract execution environment.
+    /// total pool what may in principle be transferred, reserved and used for tipping.
+    ///
+    /// This is the only balance that matters in terms of most operations on tokens. It
+    /// alone is used to determine the balance when in the contract execution environment.
     pub free: Balance,
     /// Balance which is reserved and may not be used at all.
-	///
-	/// This can still get slashed, but gets slashed last of all.
-	///
-	/// This balance is a 'reserve' balance that other subsystems use in order to set aside tokens
-	/// that are still 'owned' by the account holder, but which are suspendable.
+    ///
+    /// This can still get slashed, but gets slashed last of all.
+    ///
+    /// This balance is a 'reserve' balance that other subsystems use in order to set aside tokens
+    /// that are still 'owned' by the account holder, but which are suspendable.
     pub reserved: Balance,
     /// The amount that `free` may not drop below when withdrawing for *anything except transaction
-	/// fee payment*.
+    /// fee payment*.
     pub misc_frozen: Balance,
     /// The amount that `free` may not drop below when withdrawing specifically for transaction
-	/// fee payment.
+    /// fee payment.
     pub fee_frozen: Balance,
 }
 
@@ -139,19 +139,32 @@ pub struct AccountInfoGen<Index, AccountData> {
     /// The number of transactions this account has sent.
     pub nonce: Index,
     /// The number of other modules that currently depend on this account's existence. The account
-	/// cannot be reaped until this is zero.
+    /// cannot be reaped until this is zero.
     pub consumers: RefCount,
     /// The number of other modules that allow this account to exist. The account may not be reaped
-	/// until this and `sufficients` are both zero.
+    /// until this and `sufficients` are both zero.
     pub providers: RefCount,
     /// The additional data that belongs to this account. Used to store the balance(s) in a lot of
-	/// chains.
+    /// chains.
     pub data: AccountData,
 }
 
 pub type AccountData = AccountDataGen<Balance>;
 pub type AccountInfo = AccountInfoGen<Index, AccountData>;
 
+pub struct StorageValue(Vec<u8>);
+impl StorageValue {
+    pub fn decode<V: Decode>(&self) -> Result<V, codec::Error> {
+        Decode::decode(&mut self.deref())
+    }
+}
+impl Deref for StorageValue {
+    type Target = [u8];
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 #[cfg(feature = "std")]
 type ApiResult<T> = Result<T, ApiClientError>;
 
@@ -272,12 +285,17 @@ where
     }
 
     pub fn get_account_info(&self, address: &AccountId) -> ApiResult<Option<AccountInfo>> {
-        let storagekey: sp_core::storage::StorageKey = self
-            .metadata
-            .storage_map_key::<AccountId, AccountInfo>("System", "Account", address.clone())?;
+        let storagekey: sp_core::storage::StorageKey =
+            self.metadata
+                .storage_map_key::<AccountId>("System", "Account", address.clone())?;
         info!("storagekey {:?}", storagekey);
         info!("storage key is: 0x{}", hex::encode(storagekey.0.clone()));
-        self.get_storage_by_key_hash(storagekey, None)
+        let info = self.get_storage_by_key_hash(storagekey, None)?;
+        if let Some(info) = info {
+            Ok(Some(info.decode()?))
+        } else {
+            Ok(None)
+        }
     }
 
     pub fn get_account_data(&self, address: &AccountId) -> ApiResult<Option<AccountData>> {
@@ -330,12 +348,12 @@ where
         Self::_get_request(self.url.clone(), jsonreq)
     }
 
-    pub fn get_storage_value<V: Decode>(
+    pub fn get_storage_value(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
         at_block: Option<Hash>,
-    ) -> ApiResult<Option<V>> {
+    ) -> ApiResult<Option<StorageValue>> {
         let storagekey = self
             .metadata
             .storage_value_key(storage_prefix, storage_key_name)?;
@@ -343,39 +361,39 @@ where
         self.get_storage_by_key_hash(storagekey, at_block)
     }
 
-    pub fn get_storage_map<K: Encode, V: Decode + Clone>(
+    pub fn get_storage_map<K: Encode>(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
         map_key: K,
         at_block: Option<Hash>,
-    ) -> ApiResult<Option<V>> {
+    ) -> ApiResult<Option<StorageValue>> {
         let storagekey =
             self.metadata
-                .storage_map_key::<K, V>(storage_prefix, storage_key_name, map_key)?;
+                .storage_map_key::<K>(storage_prefix, storage_key_name, map_key)?;
         info!("storage key is: 0x{}", hex::encode(storagekey.0.clone()));
         self.get_storage_by_key_hash(storagekey, at_block)
     }
 
     pub fn get_storage_map_key_prefix(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
     ) -> ApiResult<StorageKey> {
         self.metadata
             .storage_map_key_prefix(storage_prefix, storage_key_name)
             .map_err(|e| e.into())
     }
 
-    pub fn get_storage_double_map<K: Encode, Q: Encode, V: Decode + Clone>(
+    pub fn get_storage_double_map<K: Encode, Q: Encode>(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
         first: K,
         second: Q,
         at_block: Option<Hash>,
-    ) -> ApiResult<Option<V>> {
-        let storagekey = self.metadata.storage_double_map_key::<K, Q, V>(
+    ) -> ApiResult<Option<StorageValue>> {
+        let storagekey = self.metadata.storage_double_map_key::<K, Q>(
             storage_prefix,
             storage_key_name,
             first,
@@ -385,14 +403,14 @@ where
         self.get_storage_by_key_hash(storagekey, at_block)
     }
 
-    pub fn get_storage_by_key_hash<V: Decode>(
+    pub fn get_storage_by_key_hash(
         &self,
         key: StorageKey,
         at_block: Option<Hash>,
-    ) -> ApiResult<Option<V>> {
+    ) -> ApiResult<Option<StorageValue>> {
         let s = self.get_opaque_storage_by_key_hash(key, at_block)?;
         match s {
-            Some(storage) => Ok(Some(Decode::decode(&mut storage.as_slice())?)),
+            Some(storage) => Ok(Some(StorageValue(storage))),
             None => Ok(None),
         }
     }
@@ -413,8 +431,8 @@ where
 
     pub fn get_storage_value_proof(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
         at_block: Option<Hash>,
     ) -> ApiResult<Option<ReadProof<Hash>>> {
         let storagekey = self
@@ -426,27 +444,27 @@ where
 
     pub fn get_storage_map_proof<K: Encode, V: Decode + Clone>(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
         map_key: K,
         at_block: Option<Hash>,
     ) -> ApiResult<Option<ReadProof<Hash>>> {
         let storagekey =
             self.metadata
-                .storage_map_key::<K, V>(storage_prefix, storage_key_name, map_key)?;
+                .storage_map_key::<K>(storage_prefix, storage_key_name, map_key)?;
         info!("storage key is: 0x{}", hex::encode(storagekey.0.clone()));
         self.get_storage_proof_by_keys(vec![storagekey], at_block)
     }
 
     pub fn get_storage_double_map_proof<K: Encode, Q: Encode, V: Decode + Clone>(
         &self,
-        storage_prefix: &'static str,
-        storage_key_name: &'static str,
+        storage_prefix: &str,
+        storage_key_name: &str,
         first: K,
         second: Q,
         at_block: Option<Hash>,
     ) -> ApiResult<Option<ReadProof<Hash>>> {
-        let storagekey = self.metadata.storage_double_map_key::<K, Q, V>(
+        let storagekey = self.metadata.storage_double_map_key::<K, Q>(
             storage_prefix,
             storage_key_name,
             first,
